package aufgabe04;

import java.io.InputStream;
import java.net.URL;

import javax.swing.JOptionPane;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;

/**
 * Aufgabe 04
 * 
 * Erweitern Sie das Programm, so dass es versucht Informationen zu Ihren
 * Freunden (oder den Freunden eines anderen Profils) abzurufen. Geben Sie dann
 * etwa Vor- und Nachnamen dieser Freunde aus.
 */
public class Aufgabe04 {
	/**
	 * This method returns a Jena Model representation of an RDF file that has
	 * been retrieved from a URL.
	 * 
	 * @param urlString
	 *            to RDF file
	 * @return Jena Model representation of an RDF file
	 */
	public static Model getModelFromURL(String urlString) {
		// create default model
		Model model = ModelFactory.createDefaultModel();

		// data stream from url
		InputStream in = null;

		try {
			// try to get data stream from url

			// compose url and get stream
			URL location = new URL(urlString);
			in = location.openStream();

			if (in == null) {
				throw new IllegalArgumentException("URL: " + location
						+ " not found");
			}
		} catch (Exception e) {
			// failed to get stream
			System.err.println("Failed to get RDF file from: " + urlString);
			System.exit(-1);
		}

		// read the RDF/XML file
		model.read(in, urlString);

		return model;
	}

	/**
	 * This method is used to return some data (e.g. e-mail adress, homepage,
	 * ...) from RDF-file at a specified URL.
	 * 
	 * @param urlString
	 *            to RDF file
	 * @return data from RDF file
	 */
	public static String getDataFromRDF(String urlString) {
		// initi data String object
		String data = "";

		// get initial model from url string
		Model model = getModelFromURL(urlString);

		// set properties for foaf:mailbox, foaf:homepage and foaf:family_name
		// nodes
		Property mbox = model.getProperty("http://xmlns.com/foaf/0.1/mbox");
		Property fName = model
				.getProperty("http://xmlns.com/foaf/0.1/family_name");
		Property homepage = model
				.getProperty("http://xmlns.com/foaf/0.1/homepage");

		// init iterator object for Resource objects
		ResIterator r;

		// get all family name nodes
		r = model.listSubjectsWithProperty(fName);

		// iterate family name nodes
		while (r.hasNext()) {
			// get foaf:family_name data
			Resource subject = r.nextResource();
			Statement stmt = subject.getProperty(fName);
			RDFNode object = stmt.getObject();

			// compose data
			data = data + object.toString() + ", ";
		}

		// get all foaf:mbox nodes
		r = model.listSubjectsWithProperty(mbox);

		// iterate all foaf:mailbox nodes
		while (r.hasNext()) {
			// get foaf:mbox data
			Resource subject = r.nextResource();
			Statement stmt = subject.getProperty(mbox);
			RDFNode object = stmt.getObject();

			// compose data
			data = data + object.toString() + ", ";
		}

		// get all foaf:homepage nodes
		r = model.listSubjectsWithProperty(homepage);

		// iterate all foaf:hompage nodes
		while (r.hasNext()) {
			// get all foaf:homepage data
			Resource subject = r.nextResource();
			Statement stmt = subject.getProperty(homepage);
			RDFNode object = stmt.getObject();

			// compose data
			data = data + object.toString();
		}

		return data;
	}

	public static void main(String[] args) {
		// get URL from popup window
		String urlString = JOptionPane.showInputDialog("Enter URL:",
				JOptionPane.OK_OPTION);

		// get initial model from url string
		Model model = getModelFromURL(urlString);

		// init porperties to retrieve foaf:name and foaf:knows nodes
		Property name = model.getProperty("http://xmlns.com/foaf/0.1/name");
		Property knows = model.getProperty("http://xmlns.com/foaf/0.1/knows");
		
		// init iterator to parse all nodes that
		// have at leat one foaf:knows nodes as sub node
		ResIterator r1 = model.listSubjectsWithProperty(knows);

		// iterate foaf:knows nodes
		while (r1.hasNext()) {
			// get foaf:name property of foaf:knows nodes
			Resource subject = r1.nextResource();
			Statement stmt = subject.getProperty(name);
			RDFNode object = stmt.getObject();

			// init iterator
			StmtIterator s = subject.listProperties(knows);

			// iterate all foaf:knows nodes
			while (s.hasNext()) {
				// get foaf:name property of foaf:knows nodes
				Statement stmt1 = s.nextStatement().getProperty(name);
				Resource subject1 = stmt1.getSubject();
				RDFNode object1 = stmt1.getObject();

				Property seeAlso = model.getProperty("http://www.w3.org/2000/01/rdf-schema#seeAlso");
				
				StmtIterator s2 = subject1.listProperties(seeAlso);

				// iterate all foaf:seeAlso nodes
				while (s2.hasNext()) {
					// get object of foaf:seeAlso nodes
					Statement stmt2 = s2.nextStatement();
					RDFNode object2 = stmt2.getObject();

					// extract rdfs:seeAlso of found foaf:Person node
					// in order to get some data from corresponding RDF file
					String friendData = getDataFromRDF(object2.toString());

					// print result
					System.out.println(object.toString() + " knows "
							+ object1.toString() + ", " + friendData);

				}
			}
		}
	}
}
